@@ -56,6 +56,7 @@
 #include "oscar.h"
 #include "peer.h"
 
+#include "glibcompat.h"
 #include "util.h"
 
 #define CHECKSUM_BUFFER_SIZE 256 * 1024
@@ -207,7 +208,7 @@ peer_oft_checksum_file(PeerConnection *conn, PurpleXfer *xfer, GSourceFunc callb
 	checksum_data->callback = callback;
 	checksum_data->size = size;
 	checksum_data->checksum = 0xffff0000;
-	checksum_data->file = fopen(purple_xfer_get_local_filename(xfer), "rb");
+	checksum_data->file = g_fopen(purple_xfer_get_local_filename(xfer), "rb");
 
 	if (checksum_data->file == NULL)
 	{
@@ -230,7 +231,7 @@ peer_oft_copy_xfer_data(PeerConnection *conn, OftFrame *frame)
 	g_free(conn->xferdata.name);
 
 	memcpy(&(conn->xferdata), frame, sizeof(OftFrame));
-	conn->xferdata.name = g_memdup(frame->name, frame->name_length);
+	conn->xferdata.name = g_memdup2(frame->name, frame->name_length);
 }
 
 /**
@@ -240,7 +241,7 @@ void
 peer_oft_close(PeerConnection *conn)
 {
 	/*
-	 * If canceled by local user, and we're receiving a file, and
+	 * If cancelled by local user, and we're receiving a file, and
 	 * we're not connected/ready then send an ICBM cancel message.
 	 */
 	if ((purple_xfer_get_status(conn->xfer) == PURPLE_XFER_STATUS_CANCEL_LOCAL) &&
