@@ -31,6 +31,7 @@
 #include "account.h"
 #include "connection.h"
 #include "debug.h"
+#include "glibcompat.h"
 #include "prefs.h"
 #include "pidginstock.h"
 #include "util.h"
@@ -349,8 +350,7 @@ pounce_row_selected_cb(GtkTreeView *tv, GtkTreePath *path,
 		gtk_tree_model_get(GTK_TREE_MODEL(pounce_dialog->treemodel), &iter,
 				PIDGIN_POUNCE_DATA, &pounce_data,
 				-1);
-		g_list_foreach(list, (GFunc)gtk_tree_path_free, NULL);
-		g_list_free(list);
+		g_list_free_full(list, (GDestroyNotify)gtk_tree_path_free);
 
 		pounces = purple_pounces_get_all();
 		for (; pounces != NULL; pounces = pounces->next) {
@@ -482,8 +482,7 @@ searchresults_callback_wrapper_cb(GtkWidget *widget, PidginNotifySearchResultsBu
 
 	button = bd->button;
 	button->callback(purple_account_get_connection(data->account), row, data->user_data);
-	g_list_foreach(row, (GFunc)g_free, NULL);
-	g_list_free(row);
+	g_list_free_full(row, (GDestroyNotify)g_free);
 }
 
 static void *
@@ -1349,9 +1348,16 @@ pidgin_notify_uri(const char *uri)
 		 * Does Konqueror have options to open in new tab
 		 * and/or current window?
 		 */
+	} else if (purple_strequal(web_browser, "firefox")) {
+		argv = g_slist_append(argv, "firefox");
+		if (place == PIDGIN_BROWSER_NEW_WINDOW) {
+			argv = g_slist_append(argv, "--new-window");
+		} else if (place == PIDGIN_BROWSER_NEW_TAB) {
+			argv = g_slist_append(argv, "--new-tab");
+		}
+		argv = g_slist_append(argv, uri_escaped);
 	} else if (purple_strequal(web_browser, "mozilla") ||
 		purple_strequal(web_browser, "mozilla-firebird") ||
-		purple_strequal(web_browser, "firefox") ||
 		purple_strequal(web_browser, "seamonkey"))
 	{
 		argv = g_slist_append(argv, (gpointer)web_browser);
@@ -1373,20 +1379,6 @@ pidgin_notify_uri(const char *uri)
 			argv_remote = g_slist_append(argv_remote,
 				(gpointer)web_browser);
 
-			/* Firefox 0.9 and higher require a "-a firefox" option
-			 * when using -remote commands. This breaks older
-			 * versions of mozilla. So we include this other handly
-			 * little string when calling firefox. If the API for
-			 * remote calls changes any more in firefox then firefox
-			 * should probably be split apart from mozilla-firebird
-			 * and mozilla... but this is good for now.
-			 */
-			if (purple_strequal(web_browser, "firefox")) {
-				argv_remote = g_slist_append(argv_remote, "-a");
-				argv_remote = g_slist_append(argv_remote,
-					"firefox");
-			}
-
 			argv_remote = g_slist_append(argv_remote, "-remote");
 			argv_remote = g_slist_append(argv_remote, uri_custom);
 		}
